unit OWire;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, XPMan, OWire_Module;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    ComboBox1: TComboBox;
    CheckBox1: TCheckBox;
    Button1: TButton;
    Button2: TButton;
    ListView1: TListView;
    StatusBar1: TStatusBar;
    XPManifest1: TXPManifest;
    procedure ComboBox1Change(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  DevCount:Integer;
  ListDevices:array of OW_SN;
implementation

{$R *.dfm}

function GetLastErrorString(Err:Integer):String;
begin
  case Err of
    OW_Err_OK       : Result:=' ';
    OW_Err_Port     : Result:='  COM ';
    OW_Err_NoDevice : Result:='   "1-Wire" ';
    OW_Err_IO       : Result:='  ';
    OW_Err_BaudRate : Result:='   ';
    OW_Err_Search   : Result:='   "1-Wire" ';
    OW_Err_BadParam : Result:='   ';
    OW_Err_NoData   : Result:=' ';
  end;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
var
  Port:String;
begin
  Port:=ComboBox1.Text;
  if OW_OpenPort(Port)<>0 then begin
    StatusBar1.Panels[0].Text:= ' '+Port+' ';
  end else begin
    StatusBar1.Panels[0].Text:= '    '+Port;
  end;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
var
  Mode:Integer;
begin
  Mode:=OW_GetMode();
  if CheckBox1.Checked = True then begin
     Mode:=Mode or OW_Mode_Invert;
  end else begin
     Mode:=Mode and (not OW_Mode_Invert);
  end;
  OW_SetMode(Mode)
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i, x:Integer;
  Str:String;
  Sensor:OW_SN;
begin
  StatusBar1.Panels[0].Text:= ' "1-Wire" ';
  ListView1.Clear;
  DevCount:=0;
  OW_SearchROM_Reset();

  while OW_SearchROM_Enum(Sensor)<>False do begin
    SetLength(ListDevices, DevCount+1);
    ListDevices[DevCount]:=Sensor;
    i:=DevCount;
    case ListDevices[i].DevType of
      $10 : Str:='DS1820 / DS18S20';
      $28 : Str:='DS18B20';
      else  Str:=' ';
    end;

    ListView1.Items.Add.Caption:=Str;

    Str:=IntToHex(ListDevices[i].DevType, 2)+'-';
    for x:=6 downto 1 do
      Str:=Str+IntToHex(ListDevices[i].Number[x], 2);

      Str:=Str+'-'+IntToHex(ListDevices[i].CRC, 2);
      if OW_GetCRC(@ListDevices[i], 7) <> ListDevices[i].CRC then
        Str:=Str+'   CRC';

      ListView1.Items.Item[i].SubItems.Add(Str);
      ListView1.Items.Item[i].SubItems.Add('');
      ListView1.Items.Item[i].SubItems.Add('');
      
      DevCount:=DevCount+1;
    end;

    StatusBar1.Panels[0].Text:=' '+IntToStr(DevCount)+' "1-Wire" ';
     
  //end;


end;

procedure TForm1.Button2Click(Sender: TObject);
var
  i:Integer;
  t:Smallint;
  Thermo:Real;
  RAM:OW_RAM;
begin

   if DevCount<=0 then begin
     StatusBar1.Panels[0].Text:=' "1-Wire" ';
     exit;
  end;

  if OW_SkipROM() and OW_ConvertT() then begin
     Sleep(800);
     for i:=0 to DevCount-1 do begin
        if (ListDevices[i].DevType = $10) or (ListDevices[i].DevType = $28) then begin
           If OW_SelectROM(ListDevices[i]) then begin
             If OW_ReadRAM(RAM) then begin
               If OW_GetCRC(@RAM, 8)=RAM.CRC then begin

                 t:=(RAM.H_Termo shl 8) or RAM.L_Termo;
                 if ListDevices[i].DevType = $28 then
                   Thermo:=t/16
                 else Thermo:=t/2;


                 ListView1.Items.Item[i].SubItems[1]:=FloatToStr(Thermo);
                 ListView1.Items.Item[i].SubItems[2]:='';
               end else begin
                 ListView1.Items.Item[i].SubItems[1]:='';
                 ListView1.Items.Item[i].SubItems[2]:=' CRC';
               end;
             end else begin
               ListView1.Items.Item[i].SubItems[1]:='';
               ListView1.Items.Item[i].SubItems[2]:=GetLastErrorString(OW_GetLastError());
             end;
           end else begin
             ListView1.Items.Item[i].SubItems[1]:='';
             ListView1.Items.Item[i].SubItems[2]:=GetLastErrorString(OW_GetLastError());
           end;
        end else begin
           ListView1.Items.Item[i].SubItems[2]:='  ';
        end;

     end;
  end else begin
    StatusBar1.Panels[0].Text:=GetLastErrorString(OW_GetLastError());
  end;

end;

end.
